@@ -17,8 +17,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Qgoda;
-$Qgoda::VERSION = '0.9.4';
-use strict;
+$Qgoda::VERSION = '0.9.5';
+use common::sense;
 
 # FIXME! This assumes that we are a top-level package. Instead,
 # inpect also __PACKAGE__ and adjust the directory accordingly.
@@ -28,7 +28,6 @@ my $package_dir = File::Spec->catdir(Cwd::abs_path(dirname __FILE__), 'Qgoda');
 use base 'Exporter';
 use vars qw(@EXPORT $VERSION);
 @EXPORT = qw($VERSION);
-$VERSION = '0.9.3';
 
 use Locale::TextDomain 1.30 qw(qgoda);
 use Locale::Messages;
@@ -57,7 +56,8 @@ use Qgoda::Asset;
 use Qgoda::Analyzer;
 use Qgoda::Builder;
 use Qgoda::Util qw(empty strip_suffix interpolate normalize_path write_file
-                   collect_defaults purify perl_class class2module);
+                   collect_defaults purify perl_class class2module trim
+                   read_file);
 use Qgoda::PluginUtils qw(load_plugins);
 
 my $qgoda;
@@ -194,11 +194,11 @@ sub dumpAssets {
 }
 
 sub dump {
-    my ($self) = @_;
+    my ($self, %options) = @_;
 
     my $data = [$self->dumpAssets];
 
-    my $format = $self->{__options}->{output_format};
+    my $format = $options{output_format};
     $format = 'JSON' if empty $format;
 
     if ('JSON' eq uc $format) {
@@ -227,13 +227,6 @@ sub watch {
 
     my $logger = $self->{__logger};
 
-    #eval { require AnyEvent::Filesys::Notify };
-    #if ($@) {
-    #    $logger->error($@);
-    #    $logger->fatal(__("You have to install AnyEvent::Filesys::Notify"
-    #                      . " in order to use the watch functionality"));
-    #}
-
     eval {
         # An initial build failure is fatal.
         $self->build(%options);
@@ -242,6 +235,8 @@ sub watch {
 
         $self->__startHelpers($config->{helpers});
 
+        $self->{__stop} = AnyEvent->condvar;
+
         $logger->debug(__x("waiting for changes in '{dir}'",
                            dir => $config->{srcdir}));
         AnyEvent::Filesys::Notify->new(
@@ -252,10 +247,25 @@ sub watch {
             filter => sub { $self->__filesysChangeFilter(@_) },
         );
 
-        AnyEvent::Loop::run;
-    };
+        my $reason = $self->{__stop}->recv;
+        $reason = __"no reason given!" if empty $reason;
 
+        $logger->info(__x("terminating on demand: {reason}",
+                          reason => $reason));
+    };
     $logger->fatal($@) if $@;
+
+    return $self;
+}
+
+sub stop {
+    my ($self, $reason) = @_;
+
+    if ($self->{__stop}) {
+        $self->{__stop}->send($reason);
+    } else {
+        die $reason;
+    }
 }
 
 sub __startHelpers {
@@ -401,44 +411,6 @@ sub printConfig {
 	return $self;
 }
 
-sub migrate {
-    my ($self) = @_;
-
-    my $from = $self->getOption('from_system');
-    die __"The option '--from-system' is mandatory!\n" if empty $from;
-
-    # Check for valid module names.  Yes, you can use an apostrophe as the
-    # separator in Perl but not allowing it here is at our discretion.
-    die __x("Invalid source system name '{software}'", software => $from)
-        unless $from =~ /^[a-z][a-z0-9_]+(?:(?:::|-)[a-z][a-z0-9_]+)*$/i;
-
-    my $module_name = 'Qgoda::Migrator::'  . $from;
-    $module_name =~ s/-/::/g;
-    $module_name = lc $module_name;
-    $module_name = ucfirst $module_name;
-    $module_name =~ s/::(.)/'::' . ucfirst $1/ge;
-
-    my $class_name = $module_name;
-    $module_name =~ s{::}{/}g;
-    $module_name .= '.pm';
-
-    eval {require $module_name};
-    if ($@) {
-        my $error = $@;
-        my $message = __x("Unsupported source system '{software}'!\nTry the"
-                          . " additional option '--verbose' for more"
-                          . " information!\n",
-                          software => $from);
-        $message .= $@ if $self->getOption('verbose');
-        die $message;
-    }
-
-    my $migrator = $class_name->new;
-    $migrator->migrate;
-
-    return $self;
-}
-
 sub init {
     my ($self, $args, %options) = @_;
 
@@ -457,8 +429,8 @@ sub _getProcessors {
     foreach my $module (@names) {
         my $class_name = 'Qgoda::Processor::' . $module;
 
-        if ($self->{__processors}->{$class_name}) {
-            push @processors, $self->{__processors}->{$class_name};
+        if ($self->getProcessor($class_name)) {
+            push @processors, $self->getProcessor($class_name);
             next;
         }
 
@@ -739,6 +711,19 @@ sub __filesysChangeFilter {
 
     my $config = $self->{__config};
 
+    if ($filename =~ /_stop$/ && -e $filename) {
+        my $srcdir = $config->{paths}->{srcdir};
+        my $relpath = File::Spec->abs2rel($filename, $srcdir);
+        if ('_stop' eq $relpath) {
+            my $reason = read_file $filename;
+            unlink $filename;
+            $self->stop(trim $reason);
+        }
+        return;
+    }
+
+    return if $self->{__stop};
+
     if ($config->ignorePath($filename, 1)) {
         my $logger = $self->{__logger};
         $logger->debug(__x("changed file '{filename}' is ignored",
@@ -773,12 +758,6 @@ sub __onFilesysChange {
     return $self;
 }
 
-sub getAnalyzers {
-    my ($self) = @_;
-
-    return $self->{__analyzers};
-}
-
 sub getBuilders {
     my ($self) = @_;
 
@@ -841,12 +820,6 @@ sub expandLink {
     return normalize_path $interpolated, $trailing_slash;
 }
 
-sub debugging {
-    my ($self) = @_;
-
-    return $self->{__options}->{__verbose};
-}
-
 sub getOption {
     my ($self, $name) = @_;
 
