@@ -17,8 +17,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 package Template::Plugin::Qgoda;
-$Template::Plugin::Qgoda::VERSION = '0.9.4';
-use strict;
+$Template::Plugin::Qgoda::VERSION = '0.9.5';
+use common::sense;
 
 use base qw(Template::Plugin);
 
@@ -40,11 +40,11 @@ use Qgoda::Util qw(collect_defaults merge_data empty read_file html_escape
 use Qgoda::Util::Date;
 use Qgoda::Builder;
 
+my $black_hole = sub {};
+
 sub new {
     my ($class, $context) = @_;
 
-    return $class if ref $class;
-
     my $get_values = sub {
         my ($assets, @fields) = @_;
 
@@ -123,11 +123,9 @@ sub new {
         my @keys;
         if ('ARRAY' eq reftype $objs) {
             @keys = (0 .. $#$objs);
-        } elsif ('HASH' eq reftype $objs) {
+        } else {
             # Make it deterministic.
             @keys = sort keys %$objs;
-        } else {
-            return [];
         }
 
         my $stash = Template::Stash->new({objs => $objs});
@@ -179,30 +177,17 @@ sub new {
     bless $self, $class;
 }
 
-sub __getContext {
-    shift->{__context};
-}
-
-sub __getStash {
-    shift->{__context}->stash;
-}
-
 sub __getAsset {
     shift->{__context}->stash->{asset};
 }
 
-sub __getConfig {
-    shift->{__context}->stash->{config};
-}
-
-sub bust_cache {
+sub bustCache {
     my ($self, $uri) = @_;
 
     return $uri if $uri !~ m{^/};
 
     my($scheme, $authority, $path, $query, $fragment) =
         $uri =~ m|(?:([^:/?#]+):)?(?://([^/?#]*))?([^?#]*)(?:\?([^#]*))?(?:#(.*))?|o;
-    return if !defined $path;
 
     require Qgoda;
     my $srcdir = Qgoda->new->config->{srcdir};
@@ -216,6 +201,14 @@ sub bust_cache {
     }
 }
 
+sub bust_cache {
+    my ($self, $uri) = @_;
+
+    warn __"bust_cache() is deprecated! Use bustCache() instead!\n";
+
+    return $self->bustCache($uri);
+}
+
 sub include {
     my ($self, $path, $overlay, $extra) = @_;
 
@@ -245,26 +238,28 @@ sub __include {
     my $relpath = File::Spec->abs2rel($path, $srcdir);
     my $asset = Qgoda::Asset->new($path, $relpath);
 
-    if ($overlay) {
-        my %overlay = %$overlay;
-        delete $overlay{path};
-        delete $overlay{relpath};
-        delete $overlay{reldir};
-        delete $overlay{view};
-        delete $overlay{chain};
-        delete $overlay{wrapper};
-        merge_data $asset, \%overlay;
-    }
+    my %overlay = %$overlay;
+    delete $overlay{path};
+    delete $overlay{relpath};
+    delete $overlay{reldir};
+    delete $overlay{view};
+    delete $overlay{chain};
+    delete $overlay{wrapper};
+    merge_data $asset, \%overlay;
 
-    $q->analyzeAssets([$asset], $extra);
-    $q->locateAsset($asset);
+    eval {
+        $q->analyzeAssets([$asset], $extra);
+        $q->locateAsset($asset);
 
-    my $builders = $q->getBuilders;
-    my $site = $q->getSite;
-    foreach my $builder (@{$builders}) {
-        $builder->processAsset($asset, $site);
-        $builder->wrapAsset($asset, $site);
-    }
+        my $builders = $q->getBuilders;
+        my $site = $q->getSite;
+        foreach my $builder (@{$builders}) {
+            $builder->processAsset($asset, $site);
+            $builder->wrapAsset($asset, $site);
+        }
+    };
+    die __x("Error including '{path}': {error}\n",
+            path => $_path, error => $@) if $@;
 
     return $asset;
 }
@@ -302,7 +297,7 @@ sub __sanitizeHashref {
             return {};
         } else {
             die __x("named arguments for '{method}()' are mandatory\n",
-                    method => $method);
+-                    method => $method);
         }
     }
 
@@ -386,7 +381,7 @@ sub link {
 sub existsLink {
     my ($self, @args) = @_;
 
-    local $SIG{__WARN__} = sub {};
+    local $SIG{__WARN__} = $black_hole;
 
     return $self->link(@args);
 }
@@ -401,7 +396,7 @@ sub xref {
         my $json = encode_json($filters);
         $json =~ s{.(.*).}{$1};
         warn "broken xref($json)\n";
-    } if (@$set > 1) {
+    } elsif (@$set > 1) {
         my $json = encode_json($filters);
         $json =~ s{.(.*).}{$1};
         warn "ambiguous xref($json)\n";
@@ -413,11 +408,28 @@ sub xref {
 sub existsXref {
     my ($self, @args) = @_;
 
-    local $SIG{__WARN__} = sub {};
+    local $SIG{__WARN__} = $black_hole;
 
     return $self->xref(@args);
 }
 
+sub xrefPost {
+    my ($self, $variable, $filters) = @_;
+    
+    $filters = $self->__sanitizeFilters($filters);
+    $filters->{type} = 'post';
+
+    return $self->xref($variable, $filters);
+}
+
+sub existsXrefPost {
+    my ($self, @args) = @_;
+
+    local $SIG{__WARN__} = $black_hole;
+
+    return $self->xrefPost(@args);
+}
+
 sub llink {
     my ($self, $filters) = @_;
 
@@ -430,7 +442,7 @@ sub llink {
 sub lexistsLink {
     my ($self, @args) = @_;
 
-    local $SIG{__WARN__} = sub {};
+    local $SIG{__WARN__} = $black_hole;
 
     return $self->llink(@args);
 }
@@ -444,32 +456,103 @@ sub lxref {
     return $self->xref($variable, $filters);
 }
 
+sub lxrefPost {
+    my ($self, $variable, $filters) = @_;
+
+    $filters = $self->__sanitizeFilters($filters);
+    $filters->{type} = 'post';
+
+    return $self->lxref($variable, $filters);
+}
+
 sub lexistsXref {
     my ($self, @args) = @_;
 
-    local $SIG{__WARN__} = sub {};
+    local $SIG{__WARN__} = $black_hole;
 
     return $self->lxref(@args);
 }
 
+sub lexistsXrefPost {
+    my ($self, @args) = @_;
+
+    local $SIG{__WARN__} = $black_hole;
+
+    return $self->lxrefPost(@args);
+}
+
 sub anchor {
     my ($self, $filters) = @_;
 
     my $href = escape_link $self->link($filters);
+    return '' if empty $href;
     my $title = html_escape $self->xref(title => $filters);
 
     return qq{<a href="$href">$title</a>};
 }
 
+sub anchorPost {
+    my ($self, $filters) = @_;
+
+    my $href = escape_link $self->linkPost($filters);
+    return '' if empty $href;
+    my $title = html_escape $self->xrefPost(title => $filters);
+
+    return qq{<a href="$href">$title</a>};
+}
+
 sub lanchor {
     my ($self, $filters) = @_;
 
     my $href = escape_link $self->llink($filters);
+    return '' if empty $href;
     my $title = html_escape $self->lxref(title => $filters);
 
     return qq{<a href="$href">$title</a>};
 }
 
+sub lanchorPost {
+    my ($self, $filters) = @_;
+
+    my $href = escape_link $self->llinkPost($filters);
+    return '' if empty $href;
+    my $title = html_escape $self->lxrefPost(title => $filters);
+
+    return qq{<a href="$href">$title</a>};
+}
+
+sub existsAnchor {
+    my ($self, $filters) = @_;
+
+    local $SIG{__WARN__} = $black_hole;
+
+    return $self->anchor($filters);
+}
+
+sub existsAnchorPost {
+    my ($self, $filters) = @_;
+
+    local $SIG{__WARN__} = $black_hole;
+
+    return $self->anchorPost($filters);
+}
+
+sub lexistsAnchor {
+    my ($self, $filters) = @_;
+
+    local $SIG{__WARN__} = $black_hole;
+
+    return $self->lanchor($filters);
+}
+
+sub lexistsAnchorPost {
+    my ($self, $filters) = @_;
+
+    local $SIG{__WARN__} = $black_hole;
+
+    return $self->lanchorPost($filters);
+}
+
 sub linkPost {
     my ($self, $filters) = @_;
 
@@ -482,7 +565,7 @@ sub linkPost {
 sub existsLinkPost {
     my ($self, @args) = @_;
 
-    local $SIG{__WARN__} = sub {};
+    local $SIG{__WARN__} = $black_hole;
 
     return $self->linkPost(@args);
 }
@@ -499,16 +582,14 @@ sub llinkPost {
 sub lexistsLinkPost {
     my ($self, @args) = @_;
 
-    local $SIG{__WARN__} = sub {};
+    local $SIG{__WARN__} = $black_hole;
 
     return $self->llinkPost(@args);
 }
 
-sub writeAsset {
+sub __writeAsset {
     my ($self, $path, $overlay, $extra) = @_;
 
-    die "usage: writeAsset(PATH, OVERLAY, KEY = VALUE, ...\n"
-        if empty $path || empty $overlay;
     $overlay = $self->__sanitizeHashref($overlay, 'include');
     $extra = $self->__sanitizeHashref($extra, 'include', 1);
 
@@ -542,14 +623,14 @@ sub clone {
     $extra->{relpath} = $asset->getRelpath;
     $extra->{path} = $asset->getPath;
     $extra->{parent} = $parent;
-    return $self->writeAsset($asset->getRelpath, $asset, $extra);
+    return $self->__writeAsset($asset->getRelpath, $asset, $extra);
 }
 
 sub strftime {
     my ($self, $format, $date, $lingua, $markup) = @_;
 
     my $time = $date =~ /^[-+]?[1-9][0-9]*$/ ? "$date" : str2time $date;
-    $time = $date if !defined $time;
+    return $date if !defined $time;
 
     $format = '%c' if empty $format;
 
@@ -569,24 +650,19 @@ sub strftime {
     return $formatted_date;
 }
 
-sub try {
-    require Carp;
-    Carp::croak("q.try is now invalid");
-}
-
 sub paginate {
     my ($self, $data) = @_;
 
-    $data = $self->__sanitizeHashref($data, 'paginate', 1);
-    die __"argument '{total}' is mandatory for paginate()\n"
+    $data = $self->__sanitizeHashref($data, 'paginate');
+    die __"argument 'total' is mandatory for paginate()\n"
         if empty $data->{total};
-    die __"argument '{total}' cannot be zero for paginate()\n"
-        if !$data->{total};
+    die __"argument 'total' must be positive for paginate()\n"
+        if $data->{total} <= 0;
 
     use integer;
 
     my $start = $data->{start} || 0;
-    my $total = $data->{total} || return {};
+    my $total = $data->{total};
     my $per_page = $data->{per_page} || 10;
     my $page0 = $start / $per_page;
     my $page = $page0 + 1;
@@ -598,8 +674,6 @@ sub paginate {
     my $location;
     if (!empty $asset->{plocation}) {
         $location = $asset->{plocation};
-    } elsif ($asset->{parent}) {
-        $location = $asset->{parent}->{location};
     } else {
         $location = $asset->{location};
     }
@@ -630,7 +704,7 @@ sub paginate {
     $previous_page = $links[$previous_page - 1] if defined $previous_page;
     $next_page = $links[$next_page - 1] if defined $next_page;
 
-    my @tabindexes = (0) x $#links;
+    my @tabindexes = (0) x @links;
     $tabindexes[$page0] = -1;
     $tabindexes[0] = -1 if !defined $previous_page;
 
@@ -711,7 +785,7 @@ sub encodeJSON {
     }
     my %allowed_options = map { $_ =>  1 } qw(max_depth max_size);
     foreach my $option (keys %$options) {
-        if (!exists $supported{$option}) {
+        if (!exists $allowed_options{$option}) {
             warn __x("the option '{option}' is not supported by encodeJSON().\n",
                      option => $option);
             next;
@@ -749,15 +823,25 @@ sub related {
     my ($self, $threshold, $filters) = @_;
 
     my $asset = $self->__getAsset;
-    my @related = map { $_->[0] } 
-                  sort { $b->[1] <=> $a->[1] }
-                  grep { $_->[1] >= $threshold } 
-                  @{$asset->{related}};
+
+    my $related = $asset->{related};
+    my $site = Qgoda->new->getSite;
+    my $sort_by = sub {
+        if ($related->{$a} != $related->{$b}) {
+            return $related->{$b} <=> $related->{$a};
+        } else {
+            return $a cmp $b;
+        }
+    };
+
+    my @related = map { $site->getAssetByRelpath($_) }
+        sort $sort_by
+        grep { $related->{$_} >= $threshold }
+        keys %{$asset->{related}};
+
 
     $filters = $self->__sanitizeFilters($filters);
 
-    my $site = Qgoda->new->getSite;
-
     return $site->filter(\@related, %$filters);
 }
 
@@ -770,6 +854,14 @@ sub lrelated {
     return $self->related($threshold, $filters);
 }
 
+sub relation {
+    my ($self, $other) = @_;
+
+    my $asset = $self->__getAsset;
+
+    return $asset->{related}->{$other->{relpath}};
+}
+
 sub time {
     return Qgoda::Util::Date->newFromEpoch;
 }
